/**
 * EasyBeans
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ENCBindingBuilder.java 4626 2009-02-18 13:36:43Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl.enc;

import java.util.Collection;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.enc.AbsENCBindingBuilder;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingException;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingHolder;
import org.ow2.util.ee.metadata.war.api.IWarDeployableMetadata;

/**
 * Utility class that helps to build the ENC Binding by analyzing metadata.
 * @author Florent BENOIT
 */
public class ENCBindingBuilder extends AbsENCBindingBuilder {

    /**
     * All class metadata are kept by this object.
     */
    private IWarDeployableMetadata warArchiveMetadata = null;

    /**
     * Constructor used only by this class/subclasses.
     * @param warArchiveMetadata the given metadata to analyze.
     */
    protected ENCBindingBuilder(final IWarDeployableMetadata warArchiveMetadata) {
        super();
        this.warArchiveMetadata = warArchiveMetadata;
    }

    /**
     * Analyze the metadata and populate the binding holder.
     * @throws ENCBindingException if the analyze is failing.
     */
    @Override
    protected void analyze() throws ENCBindingException {
        super.analyze();

        // Merge Persistence Contexts
        List<IJavaxPersistenceContext> javaxPersistenceContexts = warArchiveMetadata.getJavaxPersistencePersistenceContexts();
        if (javaxPersistenceContexts != null) {
            for (IJavaxPersistenceContext javaxPersistenceContext : javaxPersistenceContexts) {
                addPersistenceContext(javaxPersistenceContext, null);
            }
        }

        // Merge Persistence Units
        List<IJavaxPersistenceUnit> persistenceUnits = warArchiveMetadata.getJavaxPersistencePersistenceUnits();
        if (persistenceUnits != null) {
            for (IJavaxPersistenceUnit javaxPersistenceUnit : persistenceUnits) {
                addPersistenceUnit(javaxPersistenceUnit, null);
            }
        }

        // Merge Resources
        List<IJAnnotationResource> annotationResources = warArchiveMetadata.getJAnnotationResources();
        if (annotationResources != null) {
            for (IJAnnotationResource jAnnotationResource : annotationResources) {
                addResource(jAnnotationResource, null);
            }
        }

        // Merge WebService Ref
        List<IJaxwsWebServiceRef> jaxWSWebServiceRefs = warArchiveMetadata.getJaxwsWebServiceRefs();
        if (jaxWSWebServiceRefs != null) {
            for (IJaxwsWebServiceRef jaxwsWebServiceRef : jaxWSWebServiceRefs) {
                addWebServiceRef(jaxwsWebServiceRef, null);
            }
        }
    }
    /**
     * @return list of metadata to analyze.
     */
    @Override
    protected Collection<? extends ICommonClassMetadata<?, ?, ?>> getClassMetadataCollection() {
        return warArchiveMetadata.getWarClassMetadataCollection();
    }

    /**
     * Found all ENC bindings from the given set of metadata.
     * @param warDeployableMetadata the given metadata to analyze.
     * @return object that contains all bindings found for the given set of
     *         classes.
     * @throws ENCBindingException if the analyze of the set of the metadata has
     *         failed.
     */
    public static ENCBindingHolder analyze(final IWarDeployableMetadata warDeployableMetadata) throws ENCBindingException {
        ENCBindingBuilder bindingBuilder = new ENCBindingBuilder(warDeployableMetadata);
        bindingBuilder.analyze();
        return bindingBuilder.getHolder();
    }

}
